package fetch

import (
	"bufio"
	"errors"
	"os"
	"strconv"
	"strings"
)

const (
	memoryProcPath    = "/proc/meminfo"
	memoryTotalPrefix = "MemTotal: "
	memoryFreePrefix  = "MemFree: "

	memorySeparator = " | "
)

type MemoryStats struct {
	Total uint64
	Used  uint64
	Free  uint64
}

func Memory() (*MemoryStats, error) {
	// 420 = 0644 but go cries if i dont use octal here =(
	file, err := os.OpenFile(memoryProcPath, os.O_RDONLY, 0644)
	if err != nil {
		return nil, err
	}

	defer file.Close()

	scanner := bufio.NewScanner(file)
	readLines := 0

	var total, used, free uint64

	for scanner.Scan() {
		readLines++

		line := scanner.Text()

		splittedLine := strings.Split(line, " ")
		line = splittedLine[len(splittedLine)-2]

		var value uint64

		value, err = strconv.ParseUint(line, 10, 64)
		if err != nil {
			return nil, err
		}

		switch readLines {
		case 1:
			total = value / 1024
		case 2:
			continue
		case 3:
			free = value / 1024
			used = (total - free)
		default:
			return &MemoryStats{
				Total: total,
				Used:  used,
				Free:  free,
			}, nil
		}
	}

	return nil, errors.New("not enough lines in proc file")
}
